<?php

namespace FiloBlu\Esb\Setup;

use Magento\Framework\DB\AggregatedFieldDataConverter;
use Magento\Framework\DB\DataConverter\SerializedToJson;
use Magento\Framework\DB\FieldToConvert;
use Magento\Framework\DB\Query\Generator;
use Magento\Framework\DB\Select\QueryModifierFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Sales\Model\Order;

/**
 * Class UpgradeData
 * @package FiloBlu\Esb\Setup
 */
class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var AggregatedFieldDataConverter
     */
    private $aggregatedFieldConverter;

    /**
     * @var QueryModifierFactory
     */
    private $queryModifierFactory;

    /**
     * @var Generator
     */
    private $queryGenerator;

    /**
     * Constructor
     *
     * @param AggregatedFieldDataConverter $aggregatedFieldConverter
     * @param QueryModifierFactory $queryModifierFactory
     * @param Generator $queryGenerator
     */
    public function __construct(
        AggregatedFieldDataConverter $aggregatedFieldConverter,
        QueryModifierFactory $queryModifierFactory,
        Generator $queryGenerator
    )
    {
        $this->aggregatedFieldConverter = $aggregatedFieldConverter;
        $this->queryModifierFactory = $queryModifierFactory;
        $this->queryGenerator = $queryGenerator;
    }

    /**
     * {@inheritdoc}
     */
    public function upgrade(
        ModuleDataSetupInterface $setup,
        ModuleContextInterface $context
    )
    {
        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $confirmedStatus = $this->_statusFactory->create();
            $confirmedStatus->setData('status', 'confirmed');
            $confirmedStatus->setData('label', 'Confirmed');
            $confirmedStatus->save();
            $confirmedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $orderExportedStatus = $this->_statusFactory->create();
            $orderExportedStatus->setData('status', 'order_exported');
            $orderExportedStatus->setData('label', 'Order Exported');
            $orderExportedStatus->save();
            $orderExportedStatus->assignState(Order::STATE_PROCESSING, false, false);

            $paymentConfirmedStatus = $this->_statusFactory->create();
            $paymentConfirmedStatus->setData('status', 'payment_confirmed');
            $paymentConfirmedStatus->setData('label', 'Payment Confirmed');
            $paymentConfirmedStatus->save();
            $paymentConfirmedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $paymentExportedStatus = $this->_statusFactory->create();
            $paymentExportedStatus->setData('status', 'payment_exported');
            $paymentExportedStatus->setData('label', 'Payment Exported');
            $paymentExportedStatus->save();
            $paymentExportedStatus->assignState(Order::STATE_PROCESSING, false, false);

            $invoicedStatus = $this->_statusFactory->create();
            $invoicedStatus->setData('status', 'invoiced');
            $invoicedStatus->setData('label', 'Invoiced');
            $invoicedStatus->save();
            $invoicedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $shippedStatus = $this->_statusFactory->create();
            $shippedStatus->setData('status', 'shipped');
            $shippedStatus->setData('label', 'Shipped');
            $shippedStatus->save();
            $shippedStatus->assignState(Order::STATE_PROCESSING, false, true);

        }

        if (version_compare($context->getVersion(), '1.0.21') < 0) {
            $newStatus = $this->_statusFactory->create();
            $newStatus->setData('status', 'sent_to_logistics');
            $newStatus->setData('label', 'Sent to Logistics');
            $newStatus->save();
            $newStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);
        }

        if (version_compare($context->getVersion(), '1.1.9', '<')) {
            $this->convertSerializedDataToJson($setup);
        }
    }

    /**
     * Upgrade to version 1.1.9, convert the configuration data
     * (oad/general/order_data_fields and oad/general/item_data_fields in core_config_data table).
     *
     * @param ModuleDataSetupInterface $setup
     * @return void
     */
    private function convertSerializedDataToJson(ModuleDataSetupInterface $setup)
    {

        // Convert data for the configuration
        $queryModifier = $this->queryModifierFactory->create(
            'in',
            [
                'values' => [
                    'path' => [
                        'esb/endpoints/events',
                        'esb/ack/flows',
                        'esb/gwmapping/gwproducts'
                    ]
                ]
            ]
        );
        $fieldsToUpdate[] = new FieldToConvert(
            SerializedToJson::class,
            $setup->getTable('core_config_data'),
            'config_id',
            'value',
            $queryModifier
        );

        $this->aggregatedFieldConverter->convert($fieldsToUpdate, $setup->getConnection());
    }

}